/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "pokergame.h"

menu_t menu;
frame_t frame;

bool pokerGameInit(game_t *game) {
  pokergame_t *pokerGame = &game->pokerGame;

  // Load the Assets.
  pokerGameAssetsInit(&pokerGame->assets);

  // Initialize the Visual Novel Engine.
  vnSceneInit(&pokerGame->scene,
    &pokerGame->assets.font,
    &pokerGame->assets.testTexture
  );

  // Initialize the world
  pokerWorldInit(pokerGame);

  // Initialize the UI.
  pokerUiInit(pokerGame);

  // Add the first action, the game action, and then start the action queue.
  pokerGameActionStartAdd(pokerGame);
  queueNext(&pokerGame->scene.conversation.actionQueue);

  frameInit(&frame);
  frame.texture = &pokerGame->assets.testTexture;

  menuitem_t *item;
  menuInit(&menu);
  item = menuAdd(&menu);
  item->x = 0;
  item->y = 0;

  item = menuAdd(&menu);
  item->x = 1;
  item->y = 0;

  return true;
} 

void pokerGameUpdate(game_t *game) {
  pokergame_t *pokerGame;
  pokerGame = &game->pokerGame;

  // Update the VN Engine.
  vnSceneUpdate(&pokerGame->scene, &game->engine);

  // Bind the shader.
  shaderUse(&pokerGame->assets.shader);

  // Render the visual novel scene.
  vnSceneRenderWorld(&pokerGame->scene,&game->engine,&pokerGame->assets.shader);

  pokerWorldRender(&pokerGame->world, &game->engine, &pokerGame->assets);
  vnSceneRenderCharacters(&pokerGame->scene, &pokerGame->assets.shader);

  // Render the UI
  vnSceneRenderGui(&pokerGame->scene, &game->engine, &pokerGame->assets.shader);
  pokerUiRender(pokerGame);

  

  uint8_t i;
  menuUpdate(&menu, &game->engine);
  for(i = 0; i < menu.itemCount; i++) {
    menuitem_t *item = menu.items + i;
    frame.x = item->x * (FRAME_BORDER_SIZE + FRAME_BORDER_SIZE);
    frame.y = item->y * (FRAME_BORDER_SIZE + FRAME_BORDER_SIZE);
    if(menu.selected == i) frame.y -= FRAME_BORDER_SIZE;
    frameSetSize(&frame,
      item->width * (FRAME_BORDER_SIZE + FRAME_BORDER_SIZE),
      item->height * (FRAME_BORDER_SIZE + FRAME_BORDER_SIZE)
    );
    frameRender(&frame, &pokerGame->assets.shader);
  }
}

void pokerGameDispose(game_t *game) {
  pokergame_t *pokerGame;
  pokerGame = &game->pokerGame;

  //Cleanup the UI
  pokerUiDispose(pokerGame);

  // Cleanup the world
  pokerWorldDispose(pokerGame);

  // Destroy the Visual Novel engine.
  vnSceneDispose(&pokerGame->scene);

  // Unload all assets
  pokerGameAssetsDispose(&pokerGame->assets);
}

void pokerGameQueueRestack(pokergame_t *pokerGame) {
  arrayRewind(
    sizeof(pokergameactiondata_t),
    pokerGame->actionData,
    ANIMATION_QUEUE_ITEM_MAX,
    pokerGame->scene.conversation.actionQueue.current,
    pokerGame->scene.conversation.actionQueue.count
  );
  queueRestack(&pokerGame->scene.conversation.actionQueue);
}